from django.conf import settings
from django.core.cache import cache
from django.db import models
from django.db.models import signals
from django.dispatch import receiver
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class KeyedURL(models.Model):
    key = models.CharField(_('key'), max_length=40, unique=True)
    url = models.URLField(_('URL'), max_length=1000)

    class Meta:
        verbose_name = _('keyed URL')
        verbose_name_plural = _('keyed URLs')

    def __str__(self):
        return self.key

    def get_absolute_url(self):
        return self.url


@receiver(signals.post_save, sender=KeyedURL)
@receiver(signals.post_delete, sender=KeyedURL)
def _flush(instance, **kwargs):
    cache.delete_many([
        'keyed_urls:%s:%s' % (instance.key, language)
        for language, _ in settings.LANGUAGES
    ])



from modeltranslation.translator import translator, TranslationOptions

from keyed_urls.models import KeyedURL


class KeyedURLTranslationOptions(TranslationOptions):
    fields = ('url',)


translator.register(KeyedURL, KeyedURLTranslationOptions)



VERSION = (0, 1, 0)
__version__ = '.'.join(map(str, VERSION))


_none_type = 0xc0ffee


def get_url(key, language=None):
    from django.conf import settings
    from django.core.cache import cache
    from django.utils.translation import get_language, override
    from keyed_urls.models import KeyedURL

    language = language if language is not None else get_language()

    # Django 1.6 comes with trans_real.get_supported_language_variant;
    # we are being fast and cheap here.
    if language not in settings.LANGUAGES:
        language = language.split('-')[0]

    cache_key = 'keyed_urls:%s:%s' % (key, language)

    url = cache.get(cache_key)

    if url is None:
        try:
            instance = KeyedURL.objects.get(key=key)
        except KeyedURL.DoesNotExist:
            # We can be smart here and initialize the cache for all
            # languages.
            cache.set_many(dict((
                'keyed_urls:%s:%s' % (key, language),
                _none_type,
                ) for language, _ in settings.LANGUAGES
            ))
            url = None

        else:
            if language:
                with override(language=language):
                    url = instance.url
            else:
                url = instance.url

            cache.set(cache_key, _none_type if url is None else url, 120)

    return None if url == _none_type else url



from __future__ import absolute_import

from django import template
from django.template import TemplateSyntaxError
from django.template.defaulttags import kwarg_re
from django.utils.encoding import smart_str

from keyed_urls import get_url


register = template.Library()


class KeyedURLNode(template.Node):
    def __init__(self, key, args, kwargs, asvar):
        self.key = key
        self.args = args
        self.kwargs = kwargs
        self.asvar = asvar

    def render(self, context):
        args = [arg.resolve(context) for arg in self.args]
        kwargs = dict([
            (smart_str(k, 'ascii'), v.resolve(context))
            for k, v in self.kwargs.items()])
        key = self.key.resolve(context)

        try:
            url = get_url(key, *args, **kwargs)
        except Exception:
            if self.asvar is None:
                raise
            url = ''

        if self.asvar:
            context[self.asvar] = url
            return u''
        else:
            return url


@register.tag
def keyed_url(parser, token):
    """
    Returns an URL for the key

    Also accepts additional arguments which are passed directly to
    ``keyed_url.get_url()``.

    Usage is simple::

        {% load keyed_urls %}
        {% keyed_url 'some_key' as url %}
        <a href="{{ url }}">bla</a>

    Or::

        {% load keyed_urls %}
        <a href="{% keyed_url 'some_key' language='en' as url %}">bla</a>
    """
    bits = token.split_contents()
    if len(bits) < 2:
        raise TemplateSyntaxError(
            "'%s' takes at least one argument" % bits[0])
    key = parser.compile_filter(bits[1])
    args = []
    kwargs = {}
    asvar = None
    bits = bits[2:]
    if len(bits) >= 2 and bits[-2] == 'as':
        asvar = bits[-1]
        bits = bits[:-2]

    if len(bits):
        for bit in bits:
            match = kwarg_re.match(bit)
            if not match:
                raise TemplateSyntaxError(
                    "Malformed arguments to keyed_url tag")
            name, value = match.groups()
            if name:
                kwargs[name] = parser.compile_filter(value)
            else:
                args.append(parser.compile_filter(value))

    return KeyedURLNode(key, args, kwargs, asvar)
